/*
 * main.c
 *
 *  Created on: Jun 8, 2020
 *      Author: Ximu Zhang
 */

/* Xilinx includes. */
#include <stdio.h>
#include "xil_printf.h"
#include "xparameters.h"
#include "xil_io.h"
#include "platform.h"
#include "xgpiops.h"
// #include "xgpio.h"
#include "xttcps.h"
#include "xscutimer.h"
#include "xscugic.h"
#include "xil_exception.h"
#include "lwip/udp.h"
#include "netif/xadapter.h"
#include "xil_cache.h"
#include "xcpBasic.h"
#include <sleep.h>

#ifndef _MATH_H_
#define _MATH_H_
#include <math.h>
#endif

#ifndef _XCP_MEASURE_CALIBRATE_H_
#define _XCP_MEASURE_CALIBRATE_H_
#include "xcp_measure_calibrate.h"
#endif

/* include FPGA library here... */
/* 
#############################################
#ifndef _CM_H_
#define _CM_H_
#include "cm.h"
#endif
#############################################
*/

/* Task control */
#define TASK_100US_CONFIG	1
#define TASK_2MS_CONFIG		1
int task_2ms_count = 0;
int task_2ms_flag = 0;
void task100usFunc();
void task2msFunc();

/* Ethernet */
// Minimizing LWIP code size
#undef LWIP_TCP
#undef LWIP_DHCP

// set up netif stuctures
static struct netif server_netif;
struct netif *echo_netif;

// Global Variables for Ethernet handling
struct ip4_addr RemoteAddr;

u32 XCP_100us_Counter = 0; // XCP

volatile int TcpFastTmrFlag;
volatile int TcpSlowTmrFlag;
volatile unsigned int eth_tmr_cnt_i;
static int tcp_fast_tmr_count = 0;
static int DetectEthLinkStatus = 0;

int start_application();
void tcp_fasttmr(void);
void tcp_slowtmr(void);
void lwip_init();
void XcpInit(void);

/* Interrupt controller */
#define INTC_DEVICE_ID			XPAR_PS7_SCUGIC_0_DEVICE_ID
#define TIMER_DEVICE_ID			XPAR_SCUTIMER_DEVICE_ID
#define INTC_BASE_ADDR			XPAR_SCUGIC_0_CPU_BASEADDR
#define INTC_DIST_BASE_ADDR		XPAR_SCUGIC_0_DIST_BASEADDR
#define TIMER_IRPT_INTR			XPAR_SCUTIMER_INTR
#define XIL_EXCEPTION_ID_INT	XIL_EXCEPTION_ID_IRQ_INT
#define	TimerLoadValue (XPAR_CPU_CORTEXA9_0_CPU_CLK_FREQ_HZ/2*0.0001) // 100us interrupt - The last number represents the period of the timer
#define eth_tmr_cnt (XPAR_CPU_CORTEXA9_0_CPU_CLK_FREQ_HZ/8/TimerLoadValue) // Counter for reading the Ethernet Status registers

static XScuGic Intc; // interrupt_test Device driver instance

/* Timer */
// ARM private timer
static XScuTimer TimerInstance;

/* PS GPIO */
XGpioPs_Config *PsGpioConfig;
XGpioPs PsGpio;
int led_status = 0;
#define LED_PORT	7

/* PL GPIO */
// XGpio PlGpio;
// XGpio_Config *PlGpioConfig;
// int pl_led_status = 0;

/* declare FPGA variable here... */
/* 
#############################################
cm cm_instance;
#############################################
*/

/* Function Declaration */
void ledInit();
void TimerIntrHandler(XScuTimer *TimerInstance);
void InterruptTaskInit();
void enableInterrupt();
/*-----------------------------------------------------------*/

int main( void )
{
	/*
	 * Test data type conversion.
	 */
//	float test1 = 1.5;
//	u32 test_res1 = floatToUFix(test1, 8);
//	printf("Floating point to fixed point test\rInput is: %f, conversion result is: %lu\r\n", test1, test_res1);
//	u32 test2 = 512;
//	float test_res2 = uFixToFloat(test2, 10);
//	printf("Fixed point to floating point test\rInput is: %lu, conversion result is: %f\r\n", test2, test_res2);
//	xil_printf("Test finishes.\r\n");
//	return 0;

	// main starts
	int status;

	init_platform();
	ledInit(); // init gpiops

	/* init FPGA here... */
	/* 
	#############################################
	status = cm_Initialize(&cm_instance, XPAR_CM_1_DEVICE_ID);
	#############################################
	*/
	if (status != XST_SUCCESS)
	{
		xil_printf("FPGA initialization failed!\r\n");
		return XST_FAILURE;
	}
	else
		xil_printf("FPGA initialization succeed!\r");

	// set up Ethernet
	struct ip4_addr ipaddr, netmask, gw /*, Remotenetmask, Remotegw*/;

	/* the mac address of the board. this should be unique per board */
	unsigned char mac_ethernet_address[] =
	{ 0x00, 0x0a, 0x35, 0x00, 0x01, 0x10 };

	/* Use the same structure for the server and the echo server */
	echo_netif = &server_netif;

	// set up timer interrupt
	InterruptTaskInit();

	/* initialize IP addresses to be used */
	//IP address of ZYNQ7000
	IP4_ADDR(&ipaddr, 192, 168, 1, 10);
	IP4_ADDR(&netmask, 255, 255, 255, 0);
	IP4_ADDR(&gw, 192, 168, 1, 1);

	//IP address of the PC
	IP4_ADDR(&RemoteAddr, 192, 168, 1, 11);

	/* Initialize the lwip for UDP */
	lwip_init();

  	/* Add network interface to the netif_list, and set it as default */
	if (!xemac_add(echo_netif, &ipaddr, &netmask, &gw, mac_ethernet_address, PLATFORM_EMAC_BASEADDR)) {
		return -1;
	}
	netif_set_default(echo_netif);

	enableInterrupt();

	/* specify that the network if is up */
	netif_set_up(echo_netif);

	/* start the application (web server, rxtest, txtest, etc..) */
	start_application();

	/* Initialize XCP */
	XcpInit();

	calibarteVarsInit();

	while (1)
	{
		// for every two tcp_fasttmr, run one tcp_slowtmr
		if (TcpFastTmrFlag)
		{
			tcp_fasttmr();
			TcpFastTmrFlag = 0;
		}
		if (TcpSlowTmrFlag)
		{
			tcp_slowtmr();
			TcpSlowTmrFlag = 0;
		}

		// 2ms task
#if TASK_2MS_CONFIG
		if (task_2ms_flag) // 2ms
		{
			task2msFunc();
			task_2ms_flag = 0;
			XcpEvent(1); // XCP DAQ event - 2ms
			xcpVarsUpdate();
		}
#endif

		// Receive packets
		xemacif_input(echo_netif);
	}

	cleanup_platform();
	return 0;
}
/*-----------------------------------------------------------*/

/* Functions */
void task100usFunc()
{
	static int pl_led_count;

	// read from FPGA
	xcpVarsRead100us();
	XCP_100us_Counter++;
	XcpEvent(0); // XCP DAQ event - 100us

	// light pl leds
	// pl_led_count++;
	// if (pl_led_count == 100)
	// {
	// 	XGpio_DiscreteWrite(&PlGpio, 1, pl_led_status);
	// 	pl_led_status++;
	// 	pl_led_count = 0;
	// }
}

void task2msFunc()
{
	static int ps_led_count = 0;

	// flash led at ps side
	ps_led_count++;
	if (ps_led_count == 100) // change light for every 200ms
	{
		led_status = ((led_status == 0) ? 1:0);
		XGpioPs_WritePin(&PsGpio, LED_PORT, led_status);
		ps_led_count = 0;
	}
}

void ledInit()
{
	int status;

	// gpio configuration
	PsGpioConfig = XGpioPs_LookupConfig(XPAR_PS7_GPIO_0_DEVICE_ID);
	status = XGpioPs_CfgInitialize(&PsGpio, PsGpioConfig, PsGpioConfig->BaseAddr);
	if(status != XST_SUCCESS)
		xil_printf("PS MIO GPIO Initialize failed!\n");
	else
		xil_printf("PS MIO GPIO Initialize succeed!\n");

	XGpioPs_SetDirectionPin(&PsGpio, LED_PORT, 1);
	XGpioPs_SetOutputEnablePin(&PsGpio, LED_PORT, 1);
	XGpioPs_WritePin(&PsGpio, LED_PORT, led_status);
	// XGpioPs_SetDirectionPin(&PsGpio, 9, 1);
	// XGpioPs_SetOutputEnablePin(&PsGpio, 9, 1);
	// XGpioPs_WritePin(&PsGpio, 9, 0);
	// XGpioPs_SetDirectionPin(&PsGpio, 14, 1);
	// XGpioPs_SetOutputEnablePin(&PsGpio, 14, 1);
	// XGpioPs_WritePin(&PsGpio, 14, 0);

	// PL GPIO
	// PlGpioConfig = XGpio_LookupConfig(XPAR_AXI_GPIO_0_DEVICE_ID);
	// status = XGpio_CfgInitialize(&PlGpio, PlGpioConfig, PlGpioConfig->BaseAddress);
	// if (status != XST_SUCCESS)
	// 	xil_printf("PL GPIO initialization failed...\r\n");
	// else
	// 	xil_printf("PL GPIO initialization succeed...\r\n");
	// XGpio_SetDataDirection(&PlGpio, 1, 0x00);
}

void TimerIntrHandler(XScuTimer *TimerInstance)
{
	if (eth_tmr_cnt_i ==  (int)(eth_tmr_cnt + 0.5))
	{
		/* we need to call tcp_fasttmr & tcp_slowtmr at intervals specified
		 * by lwIP. It is not important that the timing is absoluetly accurate.
		 */
		DetectEthLinkStatus++;
		TcpFastTmrFlag = 1;

		if (tcp_fast_tmr_count)
		{
			TcpSlowTmrFlag = 1;
			tcp_fast_tmr_count = 0;
		}
		tcp_fast_tmr_count++;

		/* For detecting Ethernet phy link status periodically */
		if (DetectEthLinkStatus == ETH_LINK_DETECT_INTERVAL) {
			eth_link_detect(echo_netif);
			DetectEthLinkStatus = 0;
			}

		eth_tmr_cnt_i = 0;
	}

	// Task 0
#if TASK_100US_CONFIG
	task100usFunc();
#endif

	// task1 counter
	if (++task_2ms_count == 20)
	{
		task_2ms_flag = 1;
		task_2ms_count = 0;
	}

	eth_tmr_cnt_i++;

	XScuTimer_ClearInterruptStatus(TimerInstance); // clear interrupt
}

void InterruptTaskInit()
{
	int status;

	/* GIC Initialize */
	XScuGic_Config *IntcConfig;
	Xil_ExceptionInit();

	// interrupt controller initialization
	IntcConfig = XScuGic_LookupConfig(INTC_DEVICE_ID);
	status = XScuGic_CfgInitialize(&Intc, IntcConfig, IntcConfig->CpuBaseAddress);
	if (status != XST_SUCCESS)
		xil_printf("GIC initialization failed...\r\n");

	Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
			(Xil_ExceptionHandler)XScuGic_InterruptHandler,
			&Intc);

	// Set up timer
	XScuTimer_Config *TimerConfigPtr;
	TimerConfigPtr = XScuTimer_LookupConfig(TIMER_DEVICE_ID);
	status = XScuTimer_CfgInitialize(&TimerInstance, TimerConfigPtr, TimerConfigPtr->BaseAddr);
	if (status != XST_SUCCESS)
		xil_printf("ScuTimer initialization failed...\r\n");

	status = XScuTimer_SelfTest(&TimerInstance);
	if (status != XST_SUCCESS)
		xil_printf("ScuTimer self test failed...\r\n");

	XScuTimer_EnableAutoReload(&TimerInstance);
	XScuTimer_LoadTimer(&TimerInstance, TimerLoadValue);

	/*
	 * Connect the device driver handler that will be called when an
	 * interrupt for the device occurs, the handler defined above performs
	 * the specific interrupt processing for the device.
	 */
	status = XScuGic_Connect(&Intc, TIMER_IRPT_INTR,
					  	  	 (Xil_ExceptionHandler)TimerIntrHandler,
					  	  	 (void *)&TimerInstance);
	XScuGic_Enable(&Intc, TIMER_IRPT_INTR);
}

void enableInterrupt()
{
	XScuTimer_EnableInterrupt(&TimerInstance); //enable interrupt on the timer
	Xil_ExceptionEnableMask(XIL_EXCEPTION_IRQ); // enable interrupts in the processor
	XScuTimer_Start(&TimerInstance); // start timer
}
